# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
# This file is part of EntryPoint
### BEGIN LICENSE
# Copyright (C) 2012 Samuel Taylor <samtaylor.uk@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

import os, sys
import datetime
import tempfile

from reportlab.platypus import SimpleDocTemplate
from reportlab.lib.pagesizes import A4, landscape
from reportlab.lib.styles import getSampleStyleSheet
from reportlab.platypus import BaseDocTemplate, Frame, PageTemplate, Paragraph, Table, TableStyle
from reportlab.pdfgen import canvas
from reportlab.lib.units import mm, cm, inch
from reportlab.lib import colors

from entrypoint.kiosk.Helpers import get_fire_printers

class NumberedCanvas(canvas.Canvas):
    def __init__(self, *args, **kwargs):
        canvas.Canvas.__init__(self, *args, **kwargs)
        self._saved_page_states = []

    def showPage(self):
        self._saved_page_states.append(dict(self.__dict__))
        self._startPage()

    def save(self):
        """add page info to each page (page x of y)"""
        num_pages = len(self._saved_page_states)
        for state in self._saved_page_states:
            self.__dict__.update(state)
            self.draw_page_number(num_pages)
            canvas.Canvas.showPage(self)
        canvas.Canvas.save(self)

    def draw_page_number(self, page_count):
        self.setFont("Helvetica", 10)
        self.drawRightString(200*mm, 24,
            "Page %d of %d" % (self._pageNumber, page_count))

class FireReport():

    def __init__(self, data):
        styles = getSampleStyleSheet()
        self.styleN = styles['Normal']
        self.styleH = styles['Heading1']
        
        self.heading = "Fire Report"
        
        self.tempFile = tempfile.NamedTemporaryFile(delete=False)

        self.pageSize = A4
                    
        self.pageWidth, self.pageHeight = self.pageSize

        self.doc = BaseDocTemplate(self.tempFile.name, pagesize=self.pageSize, topMargin=40, bottomMargin=60, leftMargin=40, rightMargin=40, title=self.heading, author="EntryPoint", showBoundary=0)
        frame = Frame(self.doc.leftMargin, self.doc.bottomMargin, self.doc.width, self.doc.height, id='normal')
        template = PageTemplate(id='test', frames=frame, onPage=self.__footer)
        self.doc.addPageTemplates([template])

        self.elements = []
        self.elements.append(Paragraph(self.heading,self.styleH))
        
        tableThatSplitsOverPages = Table(data, [6 * cm, 6 * cm, 2 * cm], repeatRows=1)
        tableThatSplitsOverPages.hAlign = 'CENTER'
        tblStyle = TableStyle([('TEXTCOLOR',(0,0),(-1,-1),colors.black),
                               ('VALIGN',(0,0),(-1,-1),'TOP'),
                               ('LINEBELOW',(0,0),(-1,-1),1,colors.black),
                               ('INNERGRID', (0,0), (-1,-1), 0.25, colors.black)])
        tblStyle.add('BACKGROUND',(0,0),(-1,0),colors.lightblue)
        tblStyle.add('BACKGROUND',(0,1),(-1,-1),colors.white)
        tableThatSplitsOverPages.setStyle(tblStyle)
        self.elements.append(tableThatSplitsOverPages)
        
        self.print_report()

    def __footer(self, canvas, doc):
        canvas.saveState()
        
        today = datetime.datetime.today()
        
        P = Paragraph("Created on %s<br />Powered by EntryPoint - www.entrypoint.org.uk" % (today.strftime('%d/%m/%Y at %H:%M:%S')),self.styleN)
        w, h = P.wrap(doc.width, doc.bottomMargin)
        P.drawOn(canvas, doc.leftMargin, 24)
        
        P = Paragraph(self.heading ,self.styleN)
        w, h = P.wrap(doc.width, doc.bottomMargin)
        P.drawOn(canvas, doc.leftMargin, (self.pageHeight - h) - 24)
          
        canvas.restoreState()
        
    def build(self):
        self.doc.build(self.elements, canvasmaker=NumberedCanvas)
        
    def print_report(self):
        self.build()
        pdf=open(self.tempFile.name, "r")
        rawPdf = pdf.read()
        pdf.close()

        for printer in get_fire_printers():
            pr=os.popen("lpr -P '"+str(printer)+"' -J 'Fire Report'", "w")
            pr.write(rawPdf)
            pr.close()
